import os
from shutil import copyfile
from zipfile import ZipFile
import json
import numpy as np
import matplotlib
# important, because this script is executed on a headless server
matplotlib.use("Agg")
import matplotlib.pyplot as plt


FOLDER = os.path.join(os.path.dirname(os.path.abspath(__file__)), "data")
SENSOR_LIST = ["head", "left_foot", "right_foot", "lower_back"]
HEADER = ["accX", "accY", "accZ", "gyroX", "gyroY", "gyroZ"]


def load_signal(subject, sensor):
    """Return the signal associated with the subject-trial pair.

    Parameters
    ----------
    subject : int
        Subject number (between 0 and 85)
    sensor : str
        sensor (in SENSOR_LIST)

    Returns
    -------
    numpy array
        Signal

    """
    assert sensor in SENSOR_LIST, "Sensor not in sensor list."
    fname = os.path.join(FOLDER, "{}_{}.csv".format(subject, sensor))
    signal = np.loadtxt(fname, skiprows=1, usecols=(
        1, 2, 3, 4, 5, 6), delimiter=",")
    return signal


def dump_signal(subject, sensor):
    """Dump in a .csv file the signal associated with the subject-sensor pair.

    Parameters
    ----------
    subject : int
        Subject number (between 0 and 85)
    sensor : str
        sensor (in SENSOR_LIST)

    Returns
    -------
    numpy array
        Signal

    """
    assert sensor in SENSOR_LIST, "Sensor not in sensor list."
    fname_src = os.path.join(FOLDER, "{}_{}.csv".format(subject, sensor))
    # fname_dst = "{}_{}.csv".format(subject, sensor)
    # copyfile(fname_src, fname_dst)
    with ZipFile('signal.zip', 'w') as myzip:
        myzip.write(fname_src, arcname="{}_{}.csv".format(subject, sensor))


def dump_plot(signal):
    """Generate and dumps plot files.

    Parameters
    ----------
    signal : numpy array
        signal to plot. Shape (n_samples, 6)
    """
    n_samples, _ = signal.shape
    tt = np.arange(n_samples) / 100

    # get limits
    acc = np.take(signal, indices=[0, 1, 2], axis=1)
    if acc.size > 0:
        acc_ylim = [acc.min()-0.1, acc.max()+0.1]
    rot = np.take(signal, indices=[3, 4, 5], axis=1)
    if rot.size > 0:
        rot_ylim = [rot.min()-20, rot.max()+20]

    for dim, dim_name in enumerate(HEADER):
        fig, ax = plt.subplots(figsize=(10, 4))
        # xlim
        ax.set_xlim(0, n_samples/100)
        # plot
        ax.plot(tt, signal[:, dim])
        # ylim
        if dim_name.startswith("acc"):
            ax.set_ylim(acc_ylim)
        elif dim_name.startswith("gyro"):
            ax.set_ylim(rot_ylim)
        # number of yticks
        plt.locator_params(axis='y', nbins=6)
        # ylabel
        ylabel = "m/s²" if dim_name.startswith("acc") else "deg/s"
        ax.set_ylabel(ylabel, fontdict={"size": 20})
        for z in ax.get_yticklabels() + ax.get_xticklabels():
            z.set_fontsize(15)

        fig.tight_layout()
        plt.savefig(dim_name + ".svg", dpi=300,
                    transparent=True, bbox_inches='tight')


if __name__ == "__main__":

    import argparse

    parser = argparse.ArgumentParser(
        description='Display information and time series for a given trial.')
    parser.add_argument('--subject', metavar='subject', type=int,
                        help='The subject identifier')
    parser.add_argument('--sensor', metavar='sensor', type=str,
                        help='The sensor identifier')
    args = parser.parse_args()

    subject, sensor = args.subject, args.sensor

    # check if the code exists.

    assert subject in list(
        range(85)), "The following subject does not exist: {}".format(subject)
    assert sensor in SENSOR_LIST, "The following sensor does not exist: {}".format(
        sensor)

    # load metadata and signal
    signal = load_signal(subject, sensor)
    # dump signal file
    dump_signal(subject, sensor)
    # dump plots
    dump_plot(signal)
